supported_filters = [
    "macAddress",
    "lastName",
    "firstName",
    "dn",
    "emailAddress",
    "userId",
    "extension",
]

supported_filter_types = [
    "contains",
    "startsWith",
    "endsWith",
    "equals",
]

group_services = [
    "Account/Authorization Codes",
    "Auto Attendant - Basic",
    "Auto Attendant - Standard",
    "Call Park",
    "Call Pickup",
    "Emergency Zones",
    "Group Paging",
    "Hunt Group",
    "Incoming Calling Plan",
    "Meet-Me Conferencing",
    "Music On Hold",
    "Outgoing Calling Plan",
    "Trunk Group",
    "Voice Messaging Group",
    "VoiceXML",
]

user_services = [
    "Advice Of Charge",
    "Alternate Numbers",
    "Anonymous Call Rejection",
    "Authentication",
    "Automatic Callback",
    "Automatic Hold/Retrieve",
    "Barge-in Exempt",
    "Basic Call Logs",
    "BroadWorks Agent",
    "BroadWorks Anywhere",
    "BroadWorks Mobility",
    "BroadWorks Receptionist - Enterprise",
    "BroadWorks Receptionist - Office",
    "BroadWorks Receptionist - Small Business",
    "BroadWorks Supervisor",
    "Busy Lamp Field",
    "Call Center - Basic",
    "Call Center Monitoring",
    "Call Center - Premium",
    "Call Center - Standard",
    "Call Forwarding Always",
    "Call Forwarding Always Secondary",
    "Call Forwarding Busy",
    "Call Forwarding No Answer",
    "Call Forwarding Not Reachable",
    "Call Forwarding Selective",
    "Calling Line ID Blocking Override",
    "Calling Line ID Delivery Blocking",
    "Calling Name Delivery",
    "Calling Name Retrieval",
    "Calling Number Delivery",
    "Calling Party Category",
    "Call Me Now",
    "Call Notify",
    "Call Recording",
    "Call Return",
    "Call Transfer",
    "Call Waiting",
    "Charge Number",
    "Classmark",
    "Client License 3",
    "CommPilot Express",
    "Communication Barring User-Control",
    "Connected Line Identification Presentation",
    "Connected Line Identification Restriction",
    "Customer Originated Trace",
    "Custom Ringback User",
    "Custom Ringback User - Call Waiting",
    "Custom Ringback User - Video",
    "Directed Call Pickup",
    "Directed Call Pickup with Barge-in",
    "Directory Number Hunting",
    "Direct Route",
    "Diversion Inhibitor",
    "Do Not Disturb",
    "Enhanced Call Logs",
    "Executive",
    "Executive-Assistant",
    "External Calling Line ID Delivery",
    "External Custom Ringback",
    "Fax Messaging",
    "Flash Call Hold",
    "Flexible Seating Guest",
    "Group Night Forwarding",
    "Hoteling Guest",
    "Hoteling Host",
    "In-Call Service Activation",
    "Integrated IM&P",
    "Intercept User",
    "Internal Calling Line ID Delivery",
    "Last Number Redial",
    "Location-Based Calling Restrictions",
    "Malicious Call Trace",
    "MobileLink",
    "Multiple Call Arrangement",
    "Music On Hold User",
    "MWI Delivery to Mobile Endpoint",
    "Number Portability Announcement",
    "N-Way Call",
    "Personal Assistant",
    "Physical Location",
    "Polycom Phone Services",
    "Pre-alerting Announcement",
    "Preferred Carrier User",
    "Prepaid",
    "Priority Alert",
    "Privacy",
    "Push to Talk",
    "Remote Office",
    "Resource Priority",
    "Route List",
    "Security Classification",
    "Selective Call Acceptance",
    "Selective Call Rejection",
    "Sequential Ring",
    "Shared Call Appearance",
    "Shared Call Appearance 10",
    "Shared Call Appearance 15",
    "Shared Call Appearance 20",
    "Shared Call Appearance 25",
    "Shared Call Appearance 30",
    "Shared Call Appearance 35",
    "Shared Call Appearance 5",
    "Silent Alerting",
    "Simultaneous Ring Personal",
    "SMDI Message Desk",
    "Speed Dial 100",
    "Speed Dial 8",
    "Third-Party MWI Control",
    "Third-Party Voice Mail Support",
    "Three-Way Call",
    "Two-Stage Dialing",
    "Video Add-On",
    "Video On Hold User",
    "Virtual On-Net Enterprise Extensions",
    "Voice Messaging User",
    "Voice Messaging User - Video",
    "Voice Portal Calling",
    "Zone Calling Restrictions",
]
